#if !defined(MAT_H)
#define MAT_H
//----------------------------------------------------------
// mat.h
//
// matrix construction helpers

namespace rt
{

//--------------------------------------------------------
// mat_trans
//
// Translation matrix.
//
class mat_trans : public D3DXMATRIX
{
public:
    mat_trans(float x, float y, float z)
        : D3DXMATRIX(1, 0, 0, 0,
                     0, 1, 0, 0,
                     0, 0, 1, 0,
                     x, y, z, 1)
    {}
    mat_trans(const D3DVECTOR &point)
        : D3DXMATRIX(1, 0, 0, 0,
                     0, 1, 0, 0,
                     0, 0, 1, 0,
                     point.x, point.y, point.z, 1)
    {}
};

//--------------------------------------------------------
// mat_rot_x
//
// Rotation around the X axis about the origin
//
class mat_rot_x : public D3DXMATRIX
{
public:
    mat_rot_x(float theta) : D3DXMATRIX()
    {
        ::D3DXMatrixRotationX(static_cast<D3DXMATRIX *>(this), theta);
    }
};

//--------------------------------------------------------
// mat_rot_y
//
// Rotation around the Y axis about the origin
//
class mat_rot_y : public D3DXMATRIX
{
public:
    mat_rot_y(float theta) : D3DXMATRIX()
    {
        ::D3DXMatrixRotationY(static_cast<D3DXMATRIX *>(this), theta);
    }
};

//--------------------------------------------------------
// mat_rot_z
//
// Rotation around the Z axis about the origin
//
class mat_rot_z : public D3DXMATRIX
{
public:
    mat_rot_z(float theta) : D3DXMATRIX()
    {
        ::D3DXMatrixRotationZ(static_cast<D3DXMATRIX *>(this), theta);
    }
};

//--------------------------------------------------------
// mat_rot_axis
//
// Rotation around the given axis about the origin
//
class mat_rot_axis : public D3DXMATRIX
{
public:
    mat_rot_axis(const D3DXVECTOR3 &axis, float theta) : D3DXMATRIX()
    {
        ::D3DXMatrixRotationAxis(static_cast<D3DXMATRIX *>(this),
            &axis, theta);
    }
};

class mat_scale : public D3DXMATRIX
{
public:
    mat_scale(float s)
        : D3DXMATRIX(s, 0, 0, 0,
                     0, s, 0, 0,
                     0, 0, s, 0,
                     0, 0, 0, 1)
    {}
    mat_scale(float sx, float sy, float sz)
        : D3DXMATRIX(sx, 0, 0, 0,
                     0, sy, 0, 0,
                     0, 0, sz, 0,
                     0, 0, 0, 1)
    {}
};

//--------------------------------------------------------
// mat_look_at
//
// View matrix with camera model parameters:
// eye -- location of camera/eye
// look_at -- location of gaze
// view_up -- orientation of the camera around gaze dir
//
class mat_look_at : public D3DXMATRIX
{
public:
    mat_look_at(const D3DXVECTOR3 &eye,
                const D3DXVECTOR3 &look_at = D3DXVECTOR3(0, 0, 0),
                const D3DXVECTOR3 &view_up = D3DXVECTOR3(0, 1, 0))
        : D3DXMATRIX()
    {
        ::D3DXMatrixLookAtLH(static_cast<D3DXMATRIX *>(this), &eye, &look_at, &view_up);
    }
};

}; // rt

#endif
